#include <iostream>
#include <functional>

// Rekursion bedeutet, dass eine Funktion sich selbst aufruft.
// Sie muss also einen Namen haben. Mit function wrapper std::function:
std::function<long(int)> fakultaet1 = [] (int n) // Vorbedingung: n >= 0
{
  return n < 1 ? 1 : n*fakultaet1(n-1);
};

// Ohne std::function: Innerhalb einer Lambda-Funktion 
// ist die Fakultaets-Funktion noch nicht vollständig bekannt, deswegen
// wird sie beim Aufruf als Argument übergeben. Nicht so schön ...
// Der Typ wird erst bei der Auswertung ermittelt, deshalb auto
// und -> long. Statt auto long anzugeben, reicht nicht, weil ja
// der übergebene Parameter func als Funktion wirken soll.
auto fakultaet2 = [](auto func, int n) -> long // Vorbedingung: n >= 0
{
  return n < 1 ? 1 : n*func(func, n-1);
};

int main()
{
  for(int i = 0; i < 5; ++i) {
    std::cout << i << "! = " << fakultaet1(i) << '\n'; 
    std::cout << i << "! = " << fakultaet2(fakultaet2, i) << '\n'; 
  }
}
