#ifndef ABLAGE_H
#define ABLAGE_H
#include <condition_variable>
#include <mutex>
#include <thread>
#include <vector>
inline std::mutex ausgabeMutex;

class Ablage {
public:
  Ablage(int platz)
    : inhalt(std::vector<int>(platz)), anzahl(0),
      lesePos(-1), schreibPos(0) // -1 als Startwert
  {} 

  int get()
  {
    std::unique_lock<std::mutex> lock(monitorMutex);
    cond.wait(lock, [&,this]{ return anzahl > 0 || beendet; });
   --anzahl;
    cond.notify_all();
    lesePos = (lesePos + 1) % inhalt.size();
    return inhalt[lesePos];                  // ohne Bedeutung, falls beendet
  }

  void put(int wert)
  {
    std::unique_lock<std::mutex> lock(monitorMutex);
    cond.wait(lock, [&,this]{ return anzahl < inhalt.size() || beendet; });
    inhalt[schreibPos] = wert;
    ++anzahl;
    schreibPos = (schreibPos + 1) % inhalt.size();
    cond.notify_all();
  }

  void beenden()                             // alle wait()-Aufrufe beenden
  {
    std::unique_lock<std::mutex> lock(monitorMutex);
    beendet = true;
    cond.notify_all();
  }

  bool istBeendet() const { return beendet; }

private:
  bool beendet{false};
  std::vector<int> inhalt;
  unsigned int anzahl{0};
  // Aufbau als Ringpuffer (FIFO)
  int lesePos{0};                           // letzte gelesene Pos (nicht unsigned, s.o.)
  int schreibPos{0};                        // nächste zu schreibende Pos
  std::mutex monitorMutex;
  std::condition_variable cond;
};
#endif

